/* 
 * shrip - A DVD Encoder 
 * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "shrip-help.h"

#include <glib/gi18n.h>

static gint
calculate_max_length (const GOptionEntry *entries)
{
  gint i, len, max_length;

  max_length = 0;

  for (i = 0; entries[i].long_name; i++)
  {
    if (entries[i].flags & G_OPTION_FLAG_HIDDEN)
      continue;

    len = g_utf8_strlen (entries[i].long_name, -1);

    if (entries[i].short_name)
      len += 4;

    if (entries[i].arg != G_OPTION_ARG_NONE && entries[i].arg_description)
      len += 1 + g_utf8_strlen (entries[i].arg_description, -1);

    max_length = MAX (max_length, len);
  }

  return max_length;
}

static void
print_entry (const GOptionEntry *entry, gint max_length)
{
  GString *str;

  if (entry->flags & G_OPTION_FLAG_HIDDEN)
    return;

  if (entry->long_name[0] == 0)
    return;

  str = g_string_new (NULL);

  if (entry->short_name)
    g_string_append_printf (str, "  -%c, --%s", entry->short_name, entry->long_name);
  else
    g_string_append_printf (str, "  --%s", entry->long_name);

  if (entry->arg_description)
    g_string_append_printf (str, "=%s", entry->arg_description);

  g_print ("%-*s %s\n", max_length + 8, str->str, entry->description ? _(entry->description) : "");

  g_string_free (str, TRUE);
}

void
print_help (const gchar *command, const gchar *param_string, const gchar *descr1, ...)
{
  const GOptionEntry *entries;
  const gchar *descr;
  va_list ap, aq;

  guint i, max_length, len;
  max_length = 0;

  g_print ("Usage:\n");
  g_print ("  %s", g_get_prgname());

  if (command)
    g_print (" %s", command);

  g_print (" [OPTIONS...]");

  if (param_string)
    g_print (" %s", param_string);
  g_print ("\n\n");

  va_start (ap, descr1);
  va_copy (aq, ap);

  descr = descr1;
  while (descr)
  {
    entries = va_arg (aq, const GOptionEntry *);
    if (entries)
    {
      len = calculate_max_length (entries);
      max_length = MAX (max_length, len);
    }
    descr = va_arg (aq, const gchar *);
  }
  va_end (aq);

  descr = descr1;
  while (descr)
  {
    entries = va_arg (ap, const GOptionEntry *);
    if (entries)
    {
      g_print ("%s\n", descr);
      for (i = 0; entries[i].long_name; i++)
        print_entry (&entries[i], max_length);
      g_print ("\n");
    }
    descr = va_arg (ap, const gchar *);
  }
  va_end (ap);
}

