/* 
 * shrip - A DVD Encoder 
 * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "shrip-encode.h"
#include "shrip-ccheck.h"
#include "shrip-list.h"
#include "shrip-xchap.h"

#include <stdlib.h>
#include <string.h>

#include <glib/gi18n.h>

static void
usage (void)
{
  g_print ("%s\n", _("Usage:"));
  g_print ("  %s <%s> [%s...]\n\n", g_get_prgname (), _("COMMAND"), _("OPTION"));
  g_print ("%s:\n", _("Commands"));
  g_print ("  encode - %s\n", _("encodes selected DVD title, audio streams and subtitles"));
  g_print ("  test   - %s\n", _("performs a compressibility test on the selected DVD title"));
  g_print ("  list   - %s\n", _("lists the content of a DVD"));
  g_print ("  xchap  - %s\n", _("extracts chapter information from a DVD"));
  g_print ("\n");
  g_print ("  %s '%s <%s> --help' %s\n\n", _("Try"), g_get_prgname (), _("COMMAND"), _("to get a list of available options"));
}

static void
init (gint *argc, gchar ***argv)
{
  if (!setlocale (LC_ALL, ""))
    g_warning ("locale not supported by C library");

  if (!argc || !(*argc) || !argv)
    g_set_prgname (_("<unknown>"));
  else
  {
    gchar *prgname;

    prgname = g_path_get_basename ((*argv)[0]);
    g_set_prgname (prgname);
    g_free (prgname);
  }
}

int
main (int argc, char *argv[])
{
  gint status = EXIT_FAILURE;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

  init (&argc, &argv);

  if (argc == 1)
  {
    usage ();
    return EXIT_FAILURE;
  }

  if (strcmp (argv[1], "encode") == 0)
    status = encode (&argc, &argv);
/*
  else if (strcmp (argv[1], "ccheck") == 0)
    status = ccheck (&argc, &argv);
*/
  else if (strcmp (argv[1], "list") == 0)
    status = list (&argc, &argv);
  else if (strcmp (argv[1], "xchap") == 0)
    status = xchap (&argc, &argv);
  else
  {
    if (argv[1][0] == '-')
    {
      if (strcmp (argv[1], "--help") != 0 && strcmp (argv[1], "-h") != 0)
      {
        g_printerr (_("Unknown option '%s'"), argv[1]);
        g_printerr ("\n\n");
      }
    }
    else
    {
      g_printerr (_("Unknown command '%s'"), argv[1]);
      g_printerr ("\n\n");
    }

    usage ();
  }

  return status;
}

