/* 
 * shrip - A DVD Encoder 
 * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "shrip-help.h"
#include "shrip-common.h"

#include <ogmdvd.h>
#include <ogmjob.h>
#include <ogmrip.h>

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>

extern gchar    *device;
extern gint      start_chap;
extern gint      end_chap;
static gchar    *output      = NULL;
static gint      ntitle      = 1;
static gboolean  help        = FALSE;
static gboolean  debug       = FALSE;
/*
static gboolean
parse_chap (const gchar *name, const gchar *value, gpointer data, GError **error)
{
  gchar *str;

  if (strlen (value) == 0)
    return FALSE;

  errno = 0;
  start_chap = strtoul (value, &str, 10);

  if (!errno && *str == '-')
    end_chap = strtoul (str + 1, &str, 10);

  if (errno == ERANGE || *str != '\0')
  {
    g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, 
        _("Cannot parse value '%s' for %s"), value, name);
    return FALSE;
  }

  if (end_chap != -1 && start_chap > end_chap)
  {
    gint swap;

    swap = start_chap;
    start_chap = end_chap;
    end_chap = swap;
  }

  return TRUE;
}
*/
static const GOptionEntry entries[] = 
{
  { "device",         0,  0, G_OPTION_ARG_CALLBACK, parse_device, N_("Specify the DVD device (default: /dev/dvd)"),        "device"      },
  { "title",          0,  0, G_OPTION_ARG_INT,      &ntitle,    N_("Select the title to encode (default: 1)"),           "id",         },
  { "chapters",       0,  0, G_OPTION_ARG_CALLBACK, parse_chap, N_("Specify which chapters to extract. (default: all)"), "start[-end]" },
  { NULL,             0,  0, 0,                     NULL,       NULL,                                                    NULL          }
};

static const GOptionEntry help_entries[] =
{
  { "help",  'h', 0, G_OPTION_ARG_NONE, &help,  N_("Show help options"),     NULL },
  { "debug",  0,  0, G_OPTION_ARG_NONE, &debug, N_("Enable debug messages"), NULL },
  { NULL,     0,  0, 0,                 NULL,   NULL,                        NULL }
};

static gint
extract_xchapters (OGMDvdTitle *title, const gchar *filename)
{
  OGMJobSpawn *spawn;
  gint result;

  spawn = ogmrip_chapters_new (title, filename);
  if (!spawn)
    return OGMJOB_RESULT_ERROR;

  ogmrip_codec_set_chapters (OGMRIP_CODEC (spawn), start_chap, end_chap);

  result = ogmjob_spawn_run (spawn, NULL);
  g_object_unref (spawn);

  return result;
}

static gboolean
check_xoptions (OGMDvdTitle *title)
{
  gchar *dirname;
  gint nchapters;

  /* 
   * Check output path 
   */

  dirname = g_path_get_dirname (output);
  if (!dirname)
  {
    g_printerr ("Unknown error.\n");
    return FALSE;
  }
  if (!g_file_test (dirname, G_FILE_TEST_IS_DIR))
  {
    if (g_file_test (dirname, G_FILE_TEST_EXISTS))
    {
      g_printerr (_("'%s' exists, but is not a directory."), dirname);
      g_printerr ("\n");
      return FALSE;
    }
    else
    {
      g_printerr (_("The directory '%s' does not exist."), dirname);
      g_printerr ("\n");
      return FALSE;
    }
  }
  if (access (dirname, W_OK) == -1)
  {
    g_printerr (_("The directory '%s' is not writable."), dirname);
    g_printerr ("\n");
    return FALSE;
  }
  if (g_path_is_absolute (dirname))
    output = g_strdup (output);
  else
  {
    gchar *cwd;

    cwd = g_get_current_dir ();
    output = g_build_filename (cwd, output, NULL);
    g_free (cwd);
  }
  g_free (dirname);

  /* 
   * Check output file 
   */

  if (g_file_test (output, G_FILE_TEST_EXISTS))
  {
    g_printerr (_("A file called '%s' already exists."), output);
    g_printerr ("\n");
    return FALSE;
  }

  /*
   * Check chapters
   */

  nchapters = ogmdvd_title_get_n_chapters (title);
  if (end_chap == -1)
    end_chap = nchapters;
  start_chap --;
  end_chap--;

  if (start_chap < 0)
  {
    g_printerr (_("The chapter number %d does not exist."), start_chap);
    g_printerr ("\n");
    return FALSE;
  }

  if (end_chap >= nchapters)
  {
    g_printerr (_("The chapter number %d does not exist."), end_chap);
    g_printerr ("\n");
    return FALSE;
  }

  return TRUE;
}

static void
load_filename_default (GKeyFile *key_file, const gchar *group_name, const gchar *key_name, gchar **value)
{
  GError *error = NULL;
  gchar *val;

  val = g_key_file_get_locale_string (key_file, group_name, key_name, "UTF8", &error);
  if (error)
    g_error_free (error);
  else
  {
    *value = g_filename_from_utf8 (val, -1, NULL, NULL, NULL);
    g_free (val);
  }
}

static void
load_defaults (const gchar *filename)
{
  GKeyFile *key_file;

  key_file = g_key_file_new ();
  if (g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, NULL))
    load_filename_default (key_file, "main", "device", &device);
  g_key_file_free (key_file);
}

static void
init (gint *argc, gchar ***argv)
{
  (*argv)[0] = "shrip xchap";

  g_type_init ();
}

gint
xchap (gint *argc, gchar ***argv)
{
  GError *error = NULL;
  gint retval = EXIT_FAILURE;
  gchar *filename;

  GOptionContext *context;
  GOptionGroup   *group;

  OGMDvdDisc  *disc    = NULL;
  OGMDvdTitle *title = NULL;

  init (argc, argv);

  filename = g_build_filename (SHRIP_CONF_DIR, "shrip.conf", NULL);
  load_defaults (filename);
  g_free (filename);

  filename = g_build_filename (g_get_home_dir (), ".shrip", NULL);
  load_defaults (filename);
  g_free (filename);

  context = g_option_context_new ("<OUTPUT>");
  g_option_context_set_help_enabled (context, FALSE);
  g_option_context_add_main_entries (context, entries, NULL);

  group = g_option_group_new ("help", _("Help Options"), _("Show help options"), NULL, NULL);
  g_option_group_add_entries (group, help_entries);
  g_option_context_add_group (context, group);

  if (!g_option_context_parse (context, argc, argv, &error))
  {
    g_option_context_free (context);
    goto cleanup;
  }
  g_option_context_free (context);

  if (help)
  {
    print_help ("xchap", _("<OUTPUT>"), _("Help Options"), help_entries, _("XChap Options"), entries, NULL);
    return EXIT_SUCCESS;
  }

  if (*argc < 3)
  {
    g_printerr ("%s\n", _("No output file specified."));
    goto cleanup;
  }

  output = (*argv)[2];

#ifdef OGMRIP_CHECK_VERSION
#if OGMRIP_CHECK_VERSION(0,11,0)
  if (debug)
    ogmjob_log_set_print_stdout (TRUE);
#endif
#endif

  disc = ogmdvd_disc_new (device, NULL);
  if (!disc || !ogmdvd_disc_open (disc, NULL))
  {
    g_printerr (_("Couldn't open DVD device: '%s'"), device);
    g_printerr ("\n");
    goto cleanup;
  }

  if (ntitle < 1 || ntitle > ogmdvd_disc_get_n_titles (disc))
  {
    g_printerr (_("The title %d does not exist."), ntitle);
    g_printerr ("\n");
    goto cleanup;
  }

  title = ogmdvd_disc_get_nth_title (disc, ntitle - 1);
  ogmdvd_disc_unref (disc);

  if (!title)
  {
    g_printerr (_("Cannot open title %d."), ntitle);
    g_printerr ("\n");
    goto cleanup;
  }

  if (!check_xoptions(title))
    goto cleanup;

  if (!extract_xchapters (title, output))
    goto cleanup;

  retval = EXIT_SUCCESS;

cleanup:
  if (title)
    ogmdvd_title_unref (title);

  if (error)
  {
    g_printerr ("%s\n", error->message);
    g_error_free (error);
  }

  return retval;
}

